
#include <iostream>
#include <vector>
#include <string>
#include <rtm/CorbaNaming.h>
#include <rtm/RTObject.h>
#include <rtm/NVUtil.h>
#include <rtm/CORBA_SeqUtil.h>
#include <rtm/CorbaConsumer.h>
#include <assert.h>

#include <iostream>
using namespace std;
using namespace RTC;

vector<string> getModuleList(CorbaNaming &naming)
{

  return vector<string>();
}


void printPortProfile(CorbaConsumer<RTObject> &obj)
{
  //PortList_var port = obj->get_ports();
  PortServiceList_var port = obj->get_ports();

  cout << "controller port num=" << port->length() << endl;
  for (unsigned int i = 0; i < port->length(); ++i) {
    PortProfile* pf = port[i]->get_port_profile();
    cout << "  port[" << i << "] name=" << pf->name << endl;
  }
}

ReturnCode_t activate(CorbaConsumer<RTObject> &obj)
{
  //ExecutionContextServiceList_var eclist;
  ExecutionContextList_var eclist;

  ReturnCode_t r = RTC_ERROR;

  //eclist = obj->get_execution_context_services();
  eclist = obj->get_owned_contexts();
  for (unsigned int i = 0; i < eclist->length(); ++i) {
    if (CORBA::is_nil(eclist[i])) continue;
    r = eclist[i]->activate_component(RTObject::_duplicate(obj._ptr()));
    if (r != RTC::RTC_OK) return r;
  }
  
  return r;
}

ReturnCode_t deactivate(CorbaConsumer<RTObject> &obj)
{
  //ExecutionContextServiceList_var eclist;
  ExecutionContextList_var eclist;  
  ReturnCode_t r = RTC_ERROR;

  //eclist = obj->get_execution_context_services();
  eclist = obj->get_owned_contexts();
  for (unsigned int i = 0; i < eclist->length(); ++i) {
    if (CORBA::is_nil(eclist[i])) continue;
    r = eclist[i]->deactivate_component(RTObject::_duplicate(obj._ptr()));
    if (r != RTC::RTC_OK) return r;
  }

  return r;
}

void disconnect(CorbaConsumer<RTObject> &obj)
{
  //PortList_var port = obj->get_ports();
  PortServiceList_var port = obj->get_ports();

  for (unsigned int i = 0; i < port->length(); ++i) {
    port[i]->disconnect_all();
  }
}


//Port_var getPort(CorbaConsumer<RTObject> &obj, const string &port_name)
PortService_var getPort(CorbaConsumer<RTObject> &obj, const string &port_name)
{
  //Port_var result = NULL;
  PortService_var result = NULL;

  //PortList_var port = obj->get_ports();
  PortServiceList_var port = obj->get_ports();
  for (unsigned int i = 0; i < port->length(); ++i) {
    PortProfile* pf = port[i]->get_port_profile();
    if (string(pf->name) == port_name) {
      result = port[i];
      break;
    }
  }

  return result;
}

bool connectPort(CorbaConsumer<RTObject> &out, 
                 const string &out_port_name,
                 CorbaConsumer<RTObject> &in,
                 const string &in_port_name)
{
  // get port
  //Port_var in_port = getPort(in, in_port_name);
  //Port_var out_port = getPort(out, out_port_name);
  PortService_var in_port = getPort(in, in_port_name);
  PortService_var out_port = getPort(out, out_port_name);
  if (CORBA::is_nil(in_port) || CORBA::is_nil(out_port))
    return false;

  static int counter = 0;
  char name[128];
  sprintf(name,"connector%d",counter);
  counter++;

  // connect ports
  ConnectorProfile cp;
  cp.connector_id = "";
  cp.name = CORBA::string_dup(name);
  cp.ports.length(2);
  cp.ports[0] = in_port;
  cp.ports[1] = out_port;

  //CORBA_SeqUtil::push_back(cp.properties, NVUtil::newNV("dataport.interface_type", "CORBA_Any"));
  //CORBA_SeqUtil::push_back(cp.properties, NVUtil::newNV("dataport.dataflow_type", "Push"));
  //CORBA_SeqUtil::push_back(cp.properties, NVUtil::newNV("dataport.subscription_type","Flush"));
  CORBA_SeqUtil::push_back(cp.properties, NVUtil::newNV("dataport.interface_type", "corba_cdr"));
  CORBA_SeqUtil::push_back(cp.properties, NVUtil::newNV("dataport.dataflow_type", "push"));
  CORBA_SeqUtil::push_back(cp.properties, NVUtil::newNV("dataport.subscription_type","flush"));

  ReturnCode_t rv = in_port->connect(cp);


  if (rv != RTC::RTC_OK) return false;

  // for debug...
  std::cout << "Connector ID: " << cp.connector_id << std::endl;
  NVUtil::dump(cp.properties);

  return true;
}


int main (int argc, char** argv)
{
  if (argc != 2) {
    printf("usage : module_connector [robot name]\n");
    exit(0);
  }
  string robot_name(argv[1]);
  

  ReturnCode_t r;


  CORBA::ORB_var orb = CORBA::ORB_init(argc, argv);
  CorbaNaming naming(orb, "localhost:2809");
  //CorbaNaming naming(orb, "localhost:2920");

  vector<string> l = getModuleList(naming);
  cout << "module list" << endl;
  for (unsigned int i = 0; i < l.size(); ++i) {
    cout << "\t" << l[i] << endl;
  }



  //Controller_Module
  CorbaConsumer<RTObject> robot;
  //robot.setObject(naming.resolve("SmartPal_controller0.rtc"));
  string controller_rtc_name = robot_name + "_controller0.rtc";
  robot.setObject(naming.resolve(controller_rtc_name.c_str()));
  cout << "deactivate=" << deactivate(robot) << endl;
  printPortProfile(robot);
  disconnect(robot);
  
  //RedundancyModule
  CorbaConsumer<RTObject> rmm;
  rmm.setObject(naming.resolve("RedundancySolutionCore0.rtc"));
  cout << "deactivate=" << deactivate(rmm) << endl;
  printPortProfile(rmm);
  disconnect(rmm);

  //TrajectoryCalculation_Module
  CorbaConsumer<RTObject> trj;
  trj.setObject(naming.resolve("TrajectoryCalculation0.rtc"));
  cout << "deactivate=" << deactivate(trj) << endl;
  printPortProfile(trj);
  disconnect(trj);

  //Planner_Module
  CorbaConsumer<RTObject> plan;
  plan.setObject(naming.resolve("Planner0.rtc"));
  cout << "deactivate=" << deactivate(plan) << endl;
  printPortProfile(plan);
  disconnect(plan);

    
  ///////////////////////////////////////////////////////////////
  // connect port
  bool rv;
  
  // Pal : SmartPal_module
  // rmm  : redundancy_module
  // trj  : TrajectoryCalculation
  // plan : Planner

  string controller_name = robot_name + "_controller0";

  // connect data port
  rv = connectPort(robot, controller_name + ".q_cur_out",     rmm, "RedundancySolutionCore0.q_cur_in");
  rv = connectPort(robot, controller_name + ".q_arm_ref_in",  rmm, "RedundancySolutionCore0.q_ref_out");
  //rv = connectPort(robot, controller_name + ".q_ref_in",  rmm, "RedundancySolutionCore0.q_ref_out");


  rv = connectPort(rmm, "RedundancySolutionCore0.xyzR_out", trj, "TrajectoryCalculation0.xyzR_cur_in");
  rv = connectPort(rmm, "RedundancySolutionCore0.xyzR_in",  trj, "TrajectoryCalculation0.xyzR_ref_out");
  
  rv = connectPort(trj, "TrajectoryCalculation0.plan_flag_out",   plan, "Planner0.flag_arm_in");
  rv = connectPort(trj, "TrajectoryCalculation0.xyzR_R_final_in",   plan, "Planner0.plan_rarm_out");
  rv = connectPort(trj, "TrajectoryCalculation0.xyzR_L_final_in",   plan, "Planner0.plan_larm_out");

  rv = connectPort(plan, "Planner0.plan_odv_out",   trj, "TrajectoryCalculation0.odv_final_in");
  rv = connectPort(plan, "Planner0.plan_relbow_out",   trj, "TrajectoryCalculation0.relbow_final_in");
  rv = connectPort(plan, "Planner0.plan_lelbow_out",   trj, "TrajectoryCalculation0.lelbow_final_in");
  rv = connectPort(plan, "Planner0.plan_waist_out",   trj, "TrajectoryCalculation0.waist_final_in");

  rv = connectPort(trj, "TrajectoryCalculation0.redun_ref_out",   rmm, "RedundancySolutionCore0.redun_ref_in");
  rv = connectPort(trj, "TrajectoryCalculation0.redun_cur_in",    rmm, "RedundancySolutionCore0.redun_cur_out");


  // connect service port
  rv = connectPort(plan, "Planner0.toRedundancy",  rmm, "RedundancySolutionCore0.toRedundancy");


  // activate
  activate(rmm);
  activate(trj);
  activate(plan);
  activate(robot);

  orb->destroy();
  exit(1);
}


