// -*- C++ -*-
/*!
 * @file  Planner.cpp
 * @brief ModuleDescription
 * @date $Date$
 *
 * $Id$
 */

#include "Planner.h"
int planning_count = 0;


//#define LOGGING
#ifdef LOGGING
std::ofstream ofs1("log_rarm_plan_out.dat");
std::ofstream ofs2("log_larm_plan_out.dat");
std::ofstream ofs3("log_odv_plan_out.dat");
std::ofstream ofs4("log_relbow_plan_out.dat");
std::ofstream ofs5("log_lelbow_plan_out.dat");
std::ofstream ofs6("log_waist_plan_out.dat");
#endif //LOGGING



// Module specification
// <rtc-template block="module_spec">
static const char* planner_spec[] =
  {
    "implementation_id", "Planner",
    "type_name",         "Planner",
    "description",       "ModuleDescription",
    "version",           "1.0.0",
    "vendor",            "VenderName",
    "category",          "Category",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "0",
    "language",          "C++",
    "lang_type",         "compile",
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
Planner::Planner(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_flag_arm_inIn("flag_arm_in", m_flag_arm_in),
    m_plan_rarm_outOut("plan_rarm_out", m_plan_rarm_out),
    m_plan_larm_outOut("plan_larm_out", m_plan_larm_out),
    m_plan_odv_outOut("plan_odv_out", m_plan_odv_out),
    m_plan_relbow_outOut("plan_relbow_out", m_plan_relbow_out),
    m_plan_lelbow_outOut("plan_lelbow_out", m_plan_lelbow_out),
    m_plan_waist_outOut("plan_waist_out", m_plan_waist_out),
    m_toRedundancyPort("toRedundancy")

    // </rtc-template>
{
  CALLBACK_CLASS_INIT(m_flag_arm_in);
}

/*!
 * @brief destructor
 */
Planner::~Planner()
{
  CALLBACK_CLASS_CLEAR(m_flag_arm_in);

}



RTC::ReturnCode_t Planner::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("flag_arm_in", m_flag_arm_inIn);

  // Set OutPort buffre
  addOutPort("plan_rarm_out", m_plan_rarm_outOut);
  addOutPort("plan_larm_out", m_plan_larm_outOut);
  addOutPort("plan_odv_out", m_plan_odv_outOut);
  addOutPort("plan_relbow_out", m_plan_relbow_outOut);
  addOutPort("plan_lelbow_out", m_plan_lelbow_outOut);
  addOutPort("plan_waist_out", m_plan_waist_outOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  m_toRedundancyPort.registerConsumer("toRedundancy", "RedundancyConfig", m_toRedundancy);
  
  // Set CORBA Service Ports
  addPort(m_toRedundancyPort);
  
  // </rtc-template>

  m_flag_arm_in.data.length(1);
  m_plan_rarm_out.data.length(13);
  m_plan_larm_out.data.length(13);
  m_plan_odv_out.data.length(4);
  m_plan_relbow_out.data.length(2);
  m_plan_lelbow_out.data.length(2);
  m_plan_waist_out.data.length(4);

  std::cout << "---------- on Initialized ----------" << std::endl;

  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t Planner::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Planner::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Planner::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t Planner::onActivated(RTC::UniqueId ec_id)
{

  CALLBACK_CLASS_SET(m_flag_arm_in);

  for(int i=0; i < 13; ++i){
    m_plan_rarm_out.data[i] = 0;
    m_plan_larm_out.data[i] = 0;
  }

  std::cout << "---------- on Activated ----------" << std::endl;

  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t Planner::onDeactivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
 /*
RTC::ReturnCode_t Planner::onExecute(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
 */
/*
RTC::ReturnCode_t Planner::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Planner::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Planner::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Planner::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Planner::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

 
void Planner::_update_m_flag_arm_in(const ConnectorInfo& info,const TimedDoubleSeq& d)
{

  if(d.data.length() == 0) return;


  //------------------------------------------------------------------------
  //  default
  //------------------------------------------------------------------------
  if(planning_count == 0){

    std::cout << "[dummy_plan] Initialize" << std::endl;
 
    m_plan_rarm_out.data[0]  =  387;
    m_plan_rarm_out.data[1]  =  -316;
    m_plan_rarm_out.data[2]  =  957;
    m_plan_rarm_out.data[3]  =  0.797;
    m_plan_rarm_out.data[4]  = -0.599;
    m_plan_rarm_out.data[5]  =  0.0774;
    m_plan_rarm_out.data[6]  =  0.162;
    m_plan_rarm_out.data[7]  =  0.0893;
    m_plan_rarm_out.data[8]  = -0.983;
    m_plan_rarm_out.data[9]  =  0.581;
    m_plan_rarm_out.data[10] =  0.796;
    m_plan_rarm_out.data[11] =  0.168;
    m_plan_rarm_out.data[12]  = 0.005;


    m_plan_larm_out.data[0]  = -69.9;
    m_plan_larm_out.data[1]  =  339;
    m_plan_larm_out.data[2]  =  777;
    m_plan_larm_out.data[3]  =  0.997;
    m_plan_larm_out.data[4]  =  -0.0704;
    m_plan_larm_out.data[5]  =  0.0362;
    m_plan_larm_out.data[6]  = -0.0713;
    m_plan_larm_out.data[7]  = -0.997;
    m_plan_larm_out.data[8]  =  0.0245;
    m_plan_larm_out.data[9]  =  0.0343;
    m_plan_larm_out.data[10] = -0.027;
    m_plan_larm_out.data[11] = -0.999;
    m_plan_larm_out.data[12] =  0.005;


    m_plan_odv_out.data[0]  = 0;
    m_plan_odv_out.data[1]  = 0;
    m_plan_odv_out.data[2]  = 0;
    m_plan_odv_out.data[3]  = 0.005;

    m_plan_relbow_out.data[0]  = 0;
    m_plan_relbow_out.data[1]  = 0.005;

    m_plan_lelbow_out.data[0]  = 0;
    m_plan_lelbow_out.data[1]  = 0.005;

    m_plan_waist_out.data[0]  = 0;
    m_plan_waist_out.data[1]  = 0;
    m_plan_waist_out.data[2]  = 0;
    m_plan_waist_out.data[3]  = 0.005;



    //数値設定
    mode = 0;               // CP
    type = RIGHT;           // 対象の腕
    isHoldOtherArm = false; // 対象でないアームは制御しない
 
    result = m_toRedundancy->selectMode(mode,type,isHoldOtherArm);


    redundancy.rightElbow   = 1;
    redundancy.leftElbow    = 1;
    redundancy.lumbarRx     = 1;
    redundancy.lumbarRy     = 1;
    redundancy.lumbarRz     = 1;
    redundancy.vehicleX     = 1;
    redundancy.vehicleY     = 1;
    redundancy.vehicleTheta = 1;

    result = m_toRedundancy->selectRedundancy(redundancy);

  }


  //------------------------------------------------------------------------
  //  CP1
  //------------------------------------------------------------------------
  else if(planning_count == 1000){
    
    std::cout << "[dummy_plan] CP1 -- main = RAMR" << std::endl;
  
    m_plan_rarm_out.data[0]  =  320;
    m_plan_rarm_out.data[1]  =  -230;
    m_plan_rarm_out.data[2]  =  850;
    m_plan_rarm_out.data[3]  =  1.0;
    m_plan_rarm_out.data[4]  =  0.0;
    m_plan_rarm_out.data[5]  =  0.0;
    m_plan_rarm_out.data[6]  =  0.0;
    m_plan_rarm_out.data[7]  =  0.0;
    m_plan_rarm_out.data[8]  = -1.0;
    m_plan_rarm_out.data[9]  =  0.0;
    m_plan_rarm_out.data[10] =  1.0;
    m_plan_rarm_out.data[11] =  0.0;
    m_plan_rarm_out.data[12]  = 5;

    
    /*
    m_plan_larm_out.data[0]  =  250;
    m_plan_larm_out.data[1]  =  280;
    m_plan_larm_out.data[2]  =  850;
    m_plan_larm_out.data[3]  =  1;
    m_plan_larm_out.data[4]  =  0;
    m_plan_larm_out.data[5]  =  0;
    m_plan_larm_out.data[6]  =  0;
    m_plan_larm_out.data[7]  = -1;
    m_plan_larm_out.data[8]  =  0;
    m_plan_larm_out.data[9]  =  0;
    m_plan_larm_out.data[10] =  0;
    m_plan_larm_out.data[11] = -1;
    m_plan_larm_out.data[12]  = 5;
    */
  }


  //------------------------------------------------------------------------
  //  WAIT
  //------------------------------------------------------------------------
  else if(planning_count == 6500){
    mode = 0;                 // CP
    type = LEFT;              // 対象の腕
    isHoldOtherArm = true;    // 対象でないアームを制御する

    result = m_toRedundancy->selectMode(mode,type,isHoldOtherArm);


    redundancy.rightElbow   = 0;
    redundancy.leftElbow    = 1;
    redundancy.lumbarRx     = 1;
    redundancy.lumbarRy     = 1;
    redundancy.lumbarRz     = 1;
    redundancy.vehicleX     = 1;
    redundancy.vehicleY     = 1;
    redundancy.vehicleTheta = 1;

    result = m_toRedundancy->selectRedundancy(redundancy);

  }


  //------------------------------------------------------------------------
  //  CP1 + HOLD1
  //------------------------------------------------------------------------
  else if(planning_count == 7000){
  
    std::cout << "[dummy_plan] CP1 + HOLD1 -- main = LARM" << std::endl;

    m_plan_larm_out.data[0]  =  250;
    m_plan_larm_out.data[1]  =  280;
    m_plan_larm_out.data[2]  =  850;
    m_plan_larm_out.data[3]  =  1;
    m_plan_larm_out.data[4]  =  0;
    m_plan_larm_out.data[5]  =  0;
    m_plan_larm_out.data[6]  =  0;
    m_plan_larm_out.data[7]  = -1;
    m_plan_larm_out.data[8]  =  0;
    m_plan_larm_out.data[9]  =  0;
    m_plan_larm_out.data[10] =  0;
    m_plan_larm_out.data[11] = -1;
    m_plan_larm_out.data[12]  = 5;

  }


  //------------------------------------------------------------------------
  //  WAIT
  //------------------------------------------------------------------------
  else if(planning_count == 12500){
    mode = 1;                 // HOLD2
    type = RIGHT;             // 対象の腕
    isHoldOtherArm = false;   // 対象でないアームは制御しない

    result = m_toRedundancy->selectMode(mode,type,isHoldOtherArm);

  }


  //------------------------------------------------------------------------
  //  HOLD2
  //------------------------------------------------------------------------
  else if(planning_count == 13000){

    std::cout << "[dummy_plan] HOLD2 -- main = RARM" << std::endl;

    m_plan_relbow_out.data[0]  = 120;
    m_plan_relbow_out.data[1]  = 5;
    
    m_plan_lelbow_out.data[0]  = -150;
    m_plan_lelbow_out.data[1]  = 5;
    
    m_plan_waist_out.data[0]  = 0;
    m_plan_waist_out.data[1]  = 0;
    m_plan_waist_out.data[2]  = 10;
    m_plan_waist_out.data[3]  = 5;
    
  }


  //------------------------------------------------------------------------
  //  WAIT
  //------------------------------------------------------------------------
  else if(planning_count == 18500){
    mode = 1;                 // HOLD2
    type = LEFT;              // 対象の腕
    isHoldOtherArm = false;   // 対象でないアームは制御しない

    result = m_toRedundancy->selectMode(mode,type,isHoldOtherArm);

  }


  //------------------------------------------------------------------------
  //  HOLD2
  //------------------------------------------------------------------------
  else if(planning_count == 19000){

    std::cout << "[dummy_plan] HOLD2 -- main = LARM" << std::endl;

    m_plan_relbow_out.data[0]  = 150;
    m_plan_relbow_out.data[1]  = 5;
    
    m_plan_lelbow_out.data[0]  = -120;
    m_plan_lelbow_out.data[1]  = 5;
    
    m_plan_waist_out.data[0]  = 0;
    m_plan_waist_out.data[1]  = 10;
    m_plan_waist_out.data[2]  = 10;
    m_plan_waist_out.data[3]  = 5;
    
  }


  //------------------------------------------------------------------------
  //  WAIT
  //------------------------------------------------------------------------
  else if(planning_count == 24500){
    mode = 0;                 // CP
    type = RIGHT;             // 対象の腕
    isHoldOtherArm = true;    // 対象でないアームを制御する

    result = m_toRedundancy->selectMode(mode,type,isHoldOtherArm);


    redundancy.rightElbow   = 0;
    redundancy.leftElbow    = 0;
    redundancy.lumbarRx     = 1;
    redundancy.lumbarRy     = 1;
    redundancy.lumbarRz     = 0;
    redundancy.vehicleX     = 1;
    redundancy.vehicleY     = 1;
    redundancy.vehicleTheta = 1;

    result = m_toRedundancy->selectRedundancy(redundancy);

  }


  //------------------------------------------------------------------------
  //  CP2
  //------------------------------------------------------------------------
  else if(planning_count == 25000){
    
    std::cout << "[dummy_plan] CP2 -- main = RAMR" << std::endl;
    
    m_plan_rarm_out.data[0]  =  357;
    m_plan_rarm_out.data[1]  =  -316;
    m_plan_rarm_out.data[2]  =  957;
    m_plan_rarm_out.data[3]  =  0.797;
    m_plan_rarm_out.data[4]  = -0.599;
    m_plan_rarm_out.data[5]  =  0.0774;
    m_plan_rarm_out.data[6]  =  0.162;
    m_plan_rarm_out.data[7]  =  0.0893;
    m_plan_rarm_out.data[8]  = -0.983;
    m_plan_rarm_out.data[9]  =  0.581;
    m_plan_rarm_out.data[10] =  0.796;
    m_plan_rarm_out.data[11] =  0.168;
    m_plan_rarm_out.data[12]  = 5;

  }


  //------------------------------------------------------------------------
  //  WAIT
  //------------------------------------------------------------------------
  else if(planning_count == 30500){
    mode = 2;                 // CP + redun
    type = DUAL;              // 対象の腕
    isHoldOtherArm = true;    // 対象でないアームを制御する

    result = m_toRedundancy->selectMode(mode,type,isHoldOtherArm);


    redundancy.rightElbow   = 0;
    redundancy.leftElbow    = 0;
    redundancy.lumbarRx     = 1;
    redundancy.lumbarRy     = 1;
    redundancy.lumbarRz     = 0;
    redundancy.vehicleX     = 1;
    redundancy.vehicleY     = 1;
    redundancy.vehicleTheta = 1;

    result = m_toRedundancy->selectRedundancy(redundancy);

  }


  //------------------------------------------------------------------------
  //  CP + redun
  //------------------------------------------------------------------------
  else if(planning_count == 31000){
    
    std::cout << "[dummy_plan] CP + redun -- main = DUAL" << std::endl;
    
    m_plan_rarm_out.data[0]  =  320;
    m_plan_rarm_out.data[1]  =  -230;
    m_plan_rarm_out.data[2]  =  850;
    m_plan_rarm_out.data[3]  =  1.0;
    m_plan_rarm_out.data[4]  =  0.0;
    m_plan_rarm_out.data[5]  =  0.0;
    m_plan_rarm_out.data[6]  =  0.0;
    m_plan_rarm_out.data[7]  =  0.0;
    m_plan_rarm_out.data[8]  = -1.0;
    m_plan_rarm_out.data[9]  =  0.0;
    m_plan_rarm_out.data[10] =  1.0;
    m_plan_rarm_out.data[11] =  0.0;
    m_plan_rarm_out.data[12]  = 5;


    m_plan_larm_out.data[0]  =  250;
    m_plan_larm_out.data[1]  =  280;
    m_plan_larm_out.data[2]  =  940;
    m_plan_larm_out.data[3]  =  1;
    m_plan_larm_out.data[4]  =  0;
    m_plan_larm_out.data[5]  =  0;
    m_plan_larm_out.data[6]  =  0;
    m_plan_larm_out.data[7]  = -1;
    m_plan_larm_out.data[8]  =  0;
    m_plan_larm_out.data[9]  =  0;
    m_plan_larm_out.data[10] =  0;
    m_plan_larm_out.data[11] = -1;
    m_plan_larm_out.data[12]  = 5;



    m_plan_relbow_out.data[0]  = 100;
    m_plan_relbow_out.data[1]  = 5;
    
    m_plan_lelbow_out.data[0]  = -160;
    m_plan_lelbow_out.data[1]  = 5;
    
    m_plan_waist_out.data[0]  = 0;
    m_plan_waist_out.data[1]  = 0;
    m_plan_waist_out.data[2]  = 10;
    m_plan_waist_out.data[3]  = 5;
  }


  //------------------------------------------------------------------------
  //  WAIT
  //------------------------------------------------------------------------
  else if(planning_count == 36500){
    mode = 0;                 // CP
    type = DUAL;              // 対象の腕
    isHoldOtherArm = true;    // 対象でないアームを制御する

    result = m_toRedundancy->selectMode(mode,type,isHoldOtherArm);


    redundancy.rightElbow   = 1;
    redundancy.leftElbow    = 1;
    redundancy.lumbarRx     = 1;
    redundancy.lumbarRy     = 0;
    redundancy.lumbarRz     = 1;
    redundancy.vehicleX     = 1;
    redundancy.vehicleY     = 1;
    redundancy.vehicleTheta = 1;

    result = m_toRedundancy->selectRedundancy(redundancy);

  }


  //------------------------------------------------------------------------
  //  CP
  //------------------------------------------------------------------------
  else if(planning_count == 37000){
    
    std::cout << "[dummy_plan] CP -- main = DUAL" << std::endl;
    
    m_plan_rarm_out.data[0]  =  320;
    m_plan_rarm_out.data[1]  =  -230;
    m_plan_rarm_out.data[2]  =  950;
    m_plan_rarm_out.data[3]  =  1.0;
    m_plan_rarm_out.data[4]  =  0.0;
    m_plan_rarm_out.data[5]  =  0.0;
    m_plan_rarm_out.data[6]  =  0.0;
    m_plan_rarm_out.data[7]  =  0.0;
    m_plan_rarm_out.data[8]  = -1.0;
    m_plan_rarm_out.data[9]  =  0.0;
    m_plan_rarm_out.data[10] =  1.0;
    m_plan_rarm_out.data[11] =  0.0;
    m_plan_rarm_out.data[12]  = 5;


    m_plan_larm_out.data[0]  =  150;
    m_plan_larm_out.data[1]  =  330;
    m_plan_larm_out.data[2]  =  900;
    m_plan_larm_out.data[3]  =  1;
    m_plan_larm_out.data[4]  =  0;
    m_plan_larm_out.data[5]  =  0;
    m_plan_larm_out.data[6]  =  0;
    m_plan_larm_out.data[7]  = -1;
    m_plan_larm_out.data[8]  =  0;
    m_plan_larm_out.data[9]  =  0;
    m_plan_larm_out.data[10] =  0;
    m_plan_larm_out.data[11] = -1;
    m_plan_larm_out.data[12]  = 5;

  }




  //------------------------------------------------------------------------
  //  STOP
  //------------------------------------------------------------------------
  else {
    m_plan_rarm_out.data[12]  = 0;
    m_plan_larm_out.data[12]  = 0;
    m_plan_odv_out.data[3]    = 0;
    m_plan_relbow_out.data[1] = 0;
    m_plan_lelbow_out.data[1] = 0;
    m_plan_waist_out.data[3]  = 0;
  }

  






#ifdef LOGGING
  /////////////// LOG ///////////////
  ofs1.precision(3);
  ofs1 << m_plan_rarm_out.data[0] << ","
       << m_plan_rarm_out.data[1] << "," 
       << m_plan_rarm_out.data[2] << ",  ,"
       << m_plan_rarm_out.data[3] << ","
       << m_plan_rarm_out.data[4] << ","
       << m_plan_rarm_out.data[5] << "," 
       << m_plan_rarm_out.data[6] << ","
       << m_plan_rarm_out.data[7] << ","
       << m_plan_rarm_out.data[8] << ","
       << m_plan_rarm_out.data[9] << ","
       << m_plan_rarm_out.data[10] << ","
       << m_plan_rarm_out.data[11] << ",---," 
       << m_plan_rarm_out.data[12] << std::endl;

  ofs2.precision(3);
  ofs2 << m_plan_larm_out.data[0] << ","
       << m_plan_larm_out.data[1] << "," 
       << m_plan_larm_out.data[2] << ",  ,"
       << m_plan_larm_out.data[3] << ","
       << m_plan_larm_out.data[4] << ","
       << m_plan_larm_out.data[5] << "," 
       << m_plan_larm_out.data[6] << ","
       << m_plan_larm_out.data[7] << ","
       << m_plan_larm_out.data[8] << ","
       << m_plan_larm_out.data[9] << ","
       << m_plan_larm_out.data[10] << ","
       << m_plan_larm_out.data[11] << ",---," 
       << m_plan_larm_out.data[12] << std::endl;

  ofs3.precision(3);
  ofs3 << m_plan_odv_out.data[0] << ","
       << m_plan_odv_out.data[1] << "," 
       << m_plan_odv_out.data[2] << ",  ,"
       << m_plan_odv_out.data[3] << std::endl;

  ofs4.precision(3);
  ofs4 << m_plan_relbow_out.data[0] << ","
       << m_plan_relbow_out.data[1] << std::endl;

  ofs5.precision(3);
  ofs5 << m_plan_lelbow_out.data[0] << ","
       << m_plan_lelbow_out.data[1] << std::endl;

  ofs6.precision(3);
  ofs6 << m_plan_waist_out.data[0] << ","
       << m_plan_waist_out.data[1] << "," 
       << m_plan_waist_out.data[2] << ",  ,"
       << m_plan_waist_out.data[3] << std::endl;
#endif //LOGGING



  /////////////// OUTPUT ///////////////

  m_plan_larm_outOut.write();
  m_plan_odv_outOut.write();
  m_plan_relbow_outOut.write();
  m_plan_lelbow_outOut.write();
  m_plan_waist_outOut.write();
  m_plan_rarm_outOut.write();

  planning_count++;  

  //  std::cout << "test" << std::endl;

}
 


extern "C"
{
 
  void PlannerInit(RTC::Manager* manager)
  {
    coil::Properties profile(planner_spec);
    manager->registerFactory(profile,
                             RTC::Create<Planner>,
                             RTC::Delete<Planner>);
  }
  
};


