/**********************************************************************
  ジョイントネーム対応表を読み込むためのライブラリ
  仮想ジョイントをベースに構築

  exp)
    m_configsets.update();  
    RTC::Properties conf = m_configsets.getConfigurationSet("configuration name");

    JointName jointname;
    jointname.setConf(conf);
**********************************************************************/


#ifndef __JOINTNAME_H__
#define __JOINTNAME_H__


// C++ & C
#include <stdio.h>
#include <iostream>
#include <string>
#include <vector>
#include <map>


// OpenRTM
#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <rtm/idl/BasicDataTypeSkel.h>
#include <rtm/idl/ExtendedDataTypesSkel.h>
#include <rtm/idl/InterfaceDataTypesSkel.h>


// MACRO
#define num 26

#define BASE_X      0
#define BASE_Y      1
#define BASE_T      2
#define WAIST1      3
#define WAIST2      4
#define WAIST3      5
#define R_SHOULDER1 6
#define R_SHOULDER2 7
#define R_SHOULDER3 8
#define R_ELBOW     9
#define R_WRIST1    10
#define R_WRIST2    11
#define R_WRIST3    12
#define L_SHOULDER1 13
#define L_SHOULDER2 14
#define L_SHOULDER3 15
#define L_ELBOW     16
#define L_WRIST1    17
#define L_WRIST2    18
#define L_WRIST3    19
#define NECK1       20
#define NECK2       21
#define NECK3       22
#define CAMERA1     23
#define CAMERA2     24
#define CAMERA3     25


// namespace
using namespace std;

namespace tohoku {
  
  class JointName
  {
  public:
    JointName();
    ~JointName();
    
    
    //--------------------------------------------------------------
    //    functions
    //--------------------------------------------------------------
    
    void initParameter();
    
    // check initialized
    bool isInit() {
      return is_init;
    }
    
    // set configuration file
    bool setConf(RTC::Properties conf);
    
    
    //------  get joint name & ratio  -----------------------------
    //
    //  from index (virtual index)
    //
    vector<string> getNameReal(const int index) {return real_joint_name[index];}
    vector<double> getRatio(const int index)    {return real_joint_ratio[index];}
    string getNameVirtual(const int index)      {return virtual_joint_name[index];}
    //
    //  from name
    //
    vector<string> getNameReal(const string name_virtual);        // virtual joint name   ->  real joint name
    vector<double> getRatio(const string name_virtual);
    string getNameVirtual(const string name_real);                // real joint name      ->  virtual joint name
    
    
    int numParts()  {return VirtualJointPartNames.size();}
    vector<string> getVirtualJointPartNames()   {return VirtualJointPartNames;}
    vector<string> getVirtualJointNamesInPart(const int index)  {return VirtualJointNamesInPart[index];}
    vector<string> getVirtualJointNamesInPart(const string part_name);
    
    
    
    //------  get other variable  ---------------------------------
    //
    string getModelFile() {return model_file;}            // path for robot model 
    int    numDOF() {return DOF;}                         // real DOF
    int    numJoints() {return (int)num;}                 // virtual DOF
    
    vector<double> getControlPointOffsetP(int LR) { return offset_p[LR]; }  // R = 0, L = 1
    vector<double> getControlPointOffsetR(int LR) { return offset_R[LR]; }
    
    
    
    
  private:
    //--------------------------------------------------------------
    //    functions
    //--------------------------------------------------------------
    
    void getConfigProperty(RTC::Properties conf, string pro_name, vector<string> &pro, vector<double> &ratio);
    void getConfigProperty(RTC::Properties conf, string pro_name, vector<double> &pro);
    
    
    //--------------------------------------------------------------
    //    variables
    //--------------------------------------------------------------
    
    bool is_init;  
    vector<string> virtual_joint_name;      // joint name of standard model
    
    
    //------  variable from configuration  -------------------------
    //
    string model_file;                        // path to real model file
    int    DOF;                               // degree of freedom (real model)
    
    vector< vector<string> > real_joint_name;
    vector< vector<double> > real_joint_ratio;     // real angle = this * virtual angle
    
    vector<double> offset_p[2];
    vector<double> offset_R[2];
    
    
    //------ joint name map  ---------------------------------------
    //
    typedef map<string, int> NameToIndexMap;
    NameToIndexMap VirtualToIndexMap;         //  virtual joint name   ->  virtual joint index  ( = MACRO )
    NameToIndexMap RealToIndexMap;            //  real joint name      ->  virtual joint index  ( = MACRO )
    
    vector<string>           VirtualJointPartNames;
    vector< vector<string> > VirtualJointNamesInPart;
    NameToIndexMap PartNameToIndexMap;
  };
  
};

#endif
